@@ -25,6 +25,7 @@
 #include <limits>
 #include <cstring>
 #include <cerrno>
+#include <numeric>
 #include "parser.h"
 #include "message.h"
 #include "thinkfan.h"
@@ -35,10 +36,110 @@
 
 namespace thinkfan {
 
-Config::Config() : num_temps_(0), fan_(nullptr) {}
+
+FanConfig::FanConfig(unique_ptr<FanDriver> &&fan_drv)
+: fan_(std::move(fan_drv))
+{}
+
+const unique_ptr<FanDriver> &FanConfig::fan() const
+{ return fan_; }
+
+void FanConfig::set_fan(unique_ptr<FanDriver> &&fan)
+{ fan_ = std::move(fan); }
 
 
-const Config *Config::read_config(const std::vector<string> &filenames)
+
+StepwiseMapping::StepwiseMapping(unique_ptr<FanDriver> &&fan_drv)
+: FanConfig(std::move(fan_drv))
+{}
+
+const vector<unique_ptr<Level>> &StepwiseMapping::levels() const
+{ return levels_; }
+
+void StepwiseMapping::init_fanspeed(const TemperatureState &ts)
+{
+	cur_lvl_ = --levels().end();
+	while (cur_lvl_ != levels().begin() && (*cur_lvl_)->down(ts))
+		cur_lvl_--;
+	fan()->set_speed(**cur_lvl_);
+}
+
+bool StepwiseMapping::set_fanspeed(const TemperatureState &ts)
+{
+	if (unlikely(cur_lvl_ != --levels().end() && (*cur_lvl_)->up(ts))) {
+		while (cur_lvl_ != --levels().end() && (*cur_lvl_)->up(ts))
+			cur_lvl_++;
+		fan()->set_speed(**cur_lvl_);
+		return true;
+	}
+	else if (unlikely(cur_lvl_ != levels().begin() && (*cur_lvl_)->down(ts))) {
+		while (cur_lvl_ != levels().begin() && (*cur_lvl_)->down(ts))
+			cur_lvl_--;
+		fan()->set_speed(**cur_lvl_);
+		tmp_sleeptime = sleeptime;
+		return true;
+	}
+	else {
+		fan()->ping_watchdog_and_depulse(**cur_lvl_);
+		return false;
+	}
+}
+
+
+void StepwiseMapping::ensure_consistency(const Config &config) const
+{
+	if (levels().size() == 0)
+		throw ConfigError("No fan levels specified.");
+
+	if (!fan())
+		throw ConfigError("No fan specified in stepwise mapping.");
+
+	for (auto &lvl : levels())
+		lvl->ensure_consistency(config);
+
+	int maxlvl = (*levels_.rbegin())->num();
+	if (dynamic_cast<const HwmonFanDriver *>(fan().get()) && maxlvl < 128)
+		error<ConfigError>(MSG_CONF_MAXLVL((*levels_.rbegin())->num()));
+	else if (dynamic_cast<const TpFanDriver *>(fan().get())
+			 && maxlvl != std::numeric_limits<int>::max()
+			 && maxlvl > 7
+			 && maxlvl != 127)
+		error<ConfigError>(MSG_CONF_TP_LVL7(maxlvl, 7));
+}
+
+
+void StepwiseMapping::add_level(unique_ptr<Level> &&level)
+{
+	if (levels_.size() > 0) {
+		const unique_ptr<Level> &last_lvl = levels_.back();
+		if (level->num() != std::numeric_limits<int>::max()
+				&& level->num() != std::numeric_limits<int>::min()
+				&& last_lvl->num() > level->num())
+			error<ConfigError>(MSG_CONF_LVLORDER);
+
+		if (last_lvl->upper_limit().size() != level->upper_limit().size())
+			error<ConfigError>(MSG_CONF_LVLORDER);
+
+		for (vector<int>::const_iterator mit = last_lvl->upper_limit().begin(), oit = level->lower_limit().begin();
+				mit != last_lvl->upper_limit().end() && oit != level->lower_limit().end();
+				++mit, ++oit)
+		{
+			if (*mit < *oit) error<ConfigError>(MSG_CONF_OVERLAP);
+		}
+	}
+
+	levels_.push_back(std::move(level));
+}
+
+
+
+
+Config::Config()
+: num_temps_(0)
+{}
+
+
+const Config *Config::read_config(const vector<string> &filenames)
 {
 	const Config *rv = nullptr;
 	for (auto it = filenames.begin(); it != filenames.end(); ++it) {
@@ -73,8 +174,7 @@ const Config *Config::try_read_config(const string &filename)
 		throw IOerror(filename + ": ", errno);
 
 #ifdef USE_YAML
-	try
-	{
+	try	{
 		YAML::Node root = YAML::Load(f_data);
 
 		// Copy the return value first. Workaround for https://github.com/vmatare/thinkfan/issues/42
@@ -119,154 +219,114 @@ const Config *Config::try_read_config(const string &filename)
 	}
 #endif //USE_YAML
 
-	// Consistency checks which require the complete config
-
-	if (rv->levels().size() == 0)
-		throw ConfigError(filename + ": No fan levels specified.");
-
-	if (!rv->fan()) {
-		log(TF_WRN) << MSG_CONF_DEFAULT_FAN << flush;
-		rv->add_fan(unique_ptr<TpFanDriver>(new TpFanDriver(DEFAULT_FAN)));
-	}
-
-	if (rv->sensors().size() < 1) {
-		log(TF_WRN) << MSG_SENSOR_DEFAULT << flush;
-		rv->add_sensor(unique_ptr<TpSensorDriver>(new TpSensorDriver(DEFAULT_SENSOR, false)));
-	}
-
-	int maxlvl = (*rv->levels_.rbegin())->num();
-	if (dynamic_cast<HwmonFanDriver *>(rv->fan()) && maxlvl < 128)
-		error<ConfigError>(MSG_CONF_MAXLVL((*rv->levels_.rbegin())->num()));
-	else if (dynamic_cast<TpFanDriver *>(rv->fan())
-			 && maxlvl != std::numeric_limits<int>::max()
-			 && maxlvl > 7
-			 && maxlvl != 127)
-		error<ConfigError>(MSG_CONF_TP_LVL7(maxlvl, 7));
+	rv->src_file = filename;
+	rv->ensure_consistency();
 
 	return rv;
 }
 
 
-Config::~Config()
+void Config::ensure_consistency() const
 {
-	delete fan_;
-	for (const SensorDriver *sensor : sensors_) delete sensor;
-	for (const Level *level : levels_) delete level;
-}
+	// Consistency checks which require the complete config
 
+	if (fan_configs().empty())
+		throw ConfigError("No fans are configured in " + src_file);
 
-bool Config::add_fan(std::unique_ptr<FanDriver> &&fan)
-{
-	if (!fan) return false;
-
-	if (fan_)
-		error<ConfigError>(MSG_CONF_FAN);
-	this->fan_ = fan.release();
-	return true;
-}
-
-
-bool Config::add_sensor(std::unique_ptr<SensorDriver> &&sensor)
-{
-	if (!sensor) return false;
-
-	num_temps_ += sensor->num_temps();
-	sensors_.push_back(sensor.release());
-	return true;
-}
-
-
-bool Config::add_level(std::unique_ptr<Level> &&level)
-{
-	if (!level) return false;
-
-	if (levels_.size() > 0) {
-		const Level *last_lvl = levels_.back();
-		if (level->num() != std::numeric_limits<int>::max()
-				&& level->num() != std::numeric_limits<int>::min()
-				&& last_lvl->num() >= level->num())
-			error<ConfigError>(MSG_CONF_LVLORDER);
-
-		if (last_lvl->upper_limit().size() != level->upper_limit().size())
-			error<ConfigError>(MSG_CONF_LVLORDER);
-
-		for (std::vector<int>::const_iterator mit = last_lvl->upper_limit().begin(), oit = level->lower_limit().begin();
-				mit != last_lvl->upper_limit().end() && oit != level->lower_limit().end();
-				++mit, ++oit)
-		{
-			if (*mit < *oit) error<ConfigError>(MSG_CONF_OVERLAP);
+	for (const unique_ptr<FanConfig> &fan_cfg : fan_configs())
+		try {
+			fan_cfg->ensure_consistency(*this);
+		} catch (ConfigError &err) {
+			err.set_filename(src_file);
+			throw;
 		}
-	}
 
-	levels_.push_back(level.release());
-	return true;
+	if (sensors().size() < 1)
+		throw ConfigError(src_file + ": " + MSG_NO_SENSOR);
 }
 
 
-FanDriver *Config::fan() const
-{ return fan_; }
+
+void Config::add_sensor(unique_ptr<SensorDriver> &&sensor)
+{
+	num_temps_ += sensor->num_temps();
+	sensors_.push_back(std::move(sensor));
+}
 
 unsigned int Config::num_temps() const
 { return num_temps_; }
 
-const std::vector<Level *> &Config::levels() const
-{ return levels_; }
-
-const std::vector<SensorDriver *> &Config::sensors() const
+const vector<unique_ptr<SensorDriver>> &Config::sensors() const
 { return sensors_; }
 
+const vector<unique_ptr<FanConfig>> &Config::fan_configs() const
+{ return temp_mappings_; }
+
+void Config::add_fan_config(unique_ptr<FanConfig> &&fan_cfg)
+{ temp_mappings_.push_back(std::move(fan_cfg)); }
+
+
+void Config::init_fans() const
+{
+	for (const unique_ptr<FanConfig> &fan_cfg : fan_configs())
+		fan_cfg->fan()->init();
+}
+
 
 
 Level::Level(int level, int lower_limit, int upper_limit)
-: Level(level, std::vector<int>(1, lower_limit), std::vector<int>(1, upper_limit))
+: Level(level, vector<int>(1, lower_limit), vector<int>(1, upper_limit))
 {}
 
 Level::Level(string level, int lower_limit, int upper_limit)
-: Level(level, std::vector<int>(1, lower_limit), std::vector<int>(1, upper_limit))
+: Level(level, vector<int>(1, lower_limit), vector<int>(1, upper_limit))
 {}
 
-Level::Level(int level, const std::vector<int> &lower_limit, const std::vector<int> &upper_limit)
+Level::Level(int level, const vector<int> &lower_limit, const vector<int> &upper_limit)
 : level_s_("level " + std::to_string(level)),
   level_n_(level),
   lower_limit_(lower_limit),
   upper_limit_(upper_limit)
 {}
 
-Level::Level(string level, const std::vector<int> &lower_limit, const std::vector<int> &upper_limit)
+Level::Level(string level, const vector<int> &lower_limit, const vector<int> &upper_limit)
 : level_s_(level),
-  level_n_(std::numeric_limits<int>::max()),
+  level_n_(string_to_int(level_s_)),
   lower_limit_(lower_limit),
   upper_limit_(upper_limit)
 {
 	if (lower_limit.size() != upper_limit.size())
 		error<ConfigError>(MSG_CONF_LIMITLEN);
 
-	for (std::vector<int>::const_iterator l_it = lower_limit.begin(), u_it = upper_limit.begin();
+	for (vector<int>::const_iterator l_it = lower_limit.begin(), u_it = upper_limit.begin();
 			l_it != lower_limit.end() && u_it != upper_limit.end();
 			++u_it, ++l_it) {
 		if (*l_it != numeric_limits<int>::max() && *l_it >= *u_it)
 			error<ConfigError>(MSG_CONF_LOWHIGH);
 	}
+}
 
+int Level::string_to_int(string &level) {
+	int rv;
 	if (level == "level auto" || level == "level disengaged" || level == "level full-speed")
-		level_n_ = std::numeric_limits<int>::min();
-	else if (sscanf(level.c_str(), "level %d", &level_n_) == 1)
-		return;
+		return std::numeric_limits<int>::min();
+	else if (sscanf(level.c_str(), "level %d", &rv) == 1)
+		return rv;
 	else try {
-		level_n_ = std::stoi(level);
-		level_s_ = "level " + level;
+		rv = std::stoi(level);
+		level = "level " + level;
 	} catch (std::out_of_range &) {
 		error<ConfigError>(MSG_CONF_LVLFORMAT(level));
 	} catch (std::invalid_argument &) {
 		error<ConfigError>(MSG_CONF_LVLFORMAT(level));
 	}
+	return rv;
 }
 
-
-const std::vector<int> &Level::lower_limit() const
+const vector<int> &Level::lower_limit() const
 { return lower_limit_; }
 
-const std::vector<int> &Level::upper_limit() const
+const vector<int> &Level::upper_limit() const
 { return upper_limit_; }
 
 const string &Level::str() const
@@ -285,48 +345,88 @@ SimpleLevel::SimpleLevel(string level, int lower_limit, int upper_limit)
 : Level(level, lower_limit, upper_limit)
 {}
 
-bool SimpleLevel::up() const
+bool SimpleLevel::up(const TemperatureState &temp_state) const
 { return *temp_state.tmax >= upper_limit().front(); }
 
-bool SimpleLevel::down() const
+bool SimpleLevel::down(const TemperatureState &temp_state) const
 { return *temp_state.tmax < lower_limit().front(); }
 
+void SimpleLevel::ensure_consistency(const Config &) const
+{}
 
 
-ComplexLevel::ComplexLevel(int level, const std::vector<int> &lower_limit, const std::vector<int> &upper_limit)
+
+ComplexLevel::ComplexLevel(int level, const vector<int> &lower_limit, const vector<int> &upper_limit)
 : Level(level, lower_limit, upper_limit)
 {}
 
 
-ComplexLevel::ComplexLevel(string level, const std::vector<int> &lower_limit, const std::vector<int> &upper_limit)
+ComplexLevel::ComplexLevel(string level, const vector<int> &lower_limit, const vector<int> &upper_limit)
 : Level(level, lower_limit, upper_limit)
 {}
 
 
-bool ComplexLevel::up() const
+bool ComplexLevel::up(const TemperatureState &temp_state) const
 {
-	std::vector<int>::const_iterator temp_it = temp_state.biased_temps().begin();
-	const int *upper_idx = upper_limit().data();
+	vector<int>::const_iterator temp_it = temp_state.biased_temps().begin();
+	auto upper_it = upper_limit().begin();
 
 	while (temp_it != temp_state.biased_temps().end())
-		if (*temp_it++ >= *upper_idx++) return true;
+		if (*temp_it++ >= *upper_it++) return true;
 
 	return false;
 }
 
 
-bool ComplexLevel::down() const
+bool ComplexLevel::down(const TemperatureState &temp_state) const
 {
-	std::vector<int>::const_iterator temp_it = temp_state.biased_temps().begin();
-	const int *lower_idx = lower_limit().data();
+	auto temp_it = temp_state.biased_temps().begin();
+	auto lower_it = lower_limit().begin();
 
-	while (temp_it != temp_state.biased_temps().end() && *temp_it < *lower_idx) {
+	while (temp_it != temp_state.biased_temps().end() && *temp_it < *lower_it) {
 		temp_it++;
-		lower_idx++;
+		lower_it++;
 	}
 
 	return temp_it == temp_state.biased_temps().end();
 }
 
 
+void ComplexLevel::ensure_consistency(const Config &cfg) const
+{
+	string limitstr;
+
+	const string restmsg = " must have the length "
+		+ std::to_string(cfg.num_temps())
+		+ " (one entry for each configured sensor)"
+	;
+
+	if (lower_limit().size() != cfg.num_temps())
+		throw ConfigError(
+			"Lower limit "
+			+ format_limit(lower_limit())
+			+ restmsg
+		);
+
+	if (upper_limit().size() != cfg.num_temps())
+		throw ConfigError(
+			"Upper limit "
+			+ format_limit(upper_limit())
+			+ restmsg
+	);
+}
+
+string ComplexLevel::format_limit(const vector<int> &limit)
+{
+	return "[" + std::accumulate(
+		std::next(limit.begin()),
+		limit.end(),
+		std::to_string(limit.front()),
+		[] (string l, int r) -> string {
+			return std::move(l) + ", " + std::to_string(r);
+		}
+	) + "]";
+}
+
+
 } /* namespace thinkfan */
