@@ -26,6 +26,7 @@
 #include <execinfo.h>
 #endif
 
+#include <array>
 #include <cstring>
 #include <sstream>
 
@@ -100,7 +101,7 @@ const string &Error::backtrace() const
 { return backtrace_; }
 
 
-const char* Error::what() const _GLIBCXX_USE_NOEXCEPT
+const char* Error::what() const noexcept
 { return msg_.c_str(); }
 
 
@@ -161,14 +162,29 @@ SyntaxError::SyntaxError(const string filename, const std::ptrdiff_t offset, con
 
 ConfigError::ConfigError(const string &reason)
 : ExpectedError(reason)
+, reason_(reason)
 {}
 
+void ConfigError::set_filename(const string &filename)
+{ msg_ = filename + ":\n" + msg_; }
+
+const string &ConfigError::filename() const
+{ return filename_; }
+
+const string &ConfigError::reason() const
+{ return reason_; }
+
+const char* ConfigError::what() const noexcept
+{ return msg_.c_str(); }
+
+
 #ifdef USE_YAML
 
 ConfigError::ConfigError(const string &filename, const YAML::Mark &mark, const string &input, const string &msg)
+: reason_(msg)
+, filename_(filename)
 {
-	msg_ += filename + ":";
-
+	msg_ = filename + ":";
 	// Another workaround for Ubuntu's libyaml-cpp0.5v5
 	if (mark.pos == -1 && mark.line == -1 && mark.column == -1) {
 		msg_ += string(" ") + msg + ".";
