@@ -57,11 +57,10 @@ std::atomic<unsigned char> tolerate_errors(0);
 std::condition_variable sleep_cond;
 std::mutex sleep_mutex;
 
-
 #ifdef USE_YAML
-std::vector<string> config_files { DEFAULT_YAML_CONFIG, DEFAULT_CONFIG };
+vector<string> config_files { DEFAULT_YAML_CONFIG, DEFAULT_CONFIG };
 #else
-std::vector<std::string> config_files { DEFAULT_CONFIG };
+vector<std::string> config_files { DEFAULT_CONFIG };
 #endif
 
 std::atomic<int> interrupted(0);
@@ -82,7 +81,7 @@ void sig_handler(int signum) {
 		sleep_cond.notify_all();
 		break;
 	case SIGUSR1:
-		log(TF_INF) << temp_state << flush;
+		log(TF_NFY) << temp_state << flush;
 		break;
 #ifndef DISABLE_BUGGER
 	case SIGSEGV:
@@ -92,39 +91,21 @@ void sig_handler(int signum) {
 	case SIGUSR2:
 		interrupted = signum;
 		sleep_cond.notify_all();
-		log(TF_INF) << "Received SIGUSR2: Re-initializing fan control." << flush;
+		log(TF_NFY) << "Received SIGUSR2: Re-initializing fan control." << flush;
 		break;
 	case SIGWINCH:
-		log(TF_INF) << "Going to sleep: Will allow sensor read errors for the next 2 loops." << flush;
-		tolerate_errors = 2;
+		tolerate_errors = 4;
+		log(TF_NFY) << "Going to sleep: Will allow sensor read errors for the next "
+			<< std::to_string(tolerate_errors) << " loops." << flush;
 	}
 }
 
 
-
-static inline void sensor_lost(const SensorDriver *s, const ExpectedError &e) {
-	if (!s->optional() || tolerate_errors)
-		error<SensorLost>(e);
-	else
-		log(TF_INF) << SensorLost(e).what();
-	temp_state.add_temp(-128);
-}
-
-
-static inline void read_temps_safe(const std::vector<SensorDriver *> &sensors)
+static inline void read_temps(const vector<unique_ptr<SensorDriver>> &sensors)
 {
 	temp_state.restart();
-	for (const SensorDriver *sensor : sensors) {
-		try {
-			sensor->read_temps();
-		} catch (SystemError &e) {
-			sensor_lost(sensor, e);
-		} catch (IOerror &e) {
-			sensor_lost(sensor, e);
-		} catch (std::ios_base::failure &e) {
-			sensor_lost(sensor, IOerror(e.what(), THINKFAN_IO_ERROR_CODE(e)));
-		}
-	}
+	for (const unique_ptr<SensorDriver> &sensor : sensors)
+		sensor->read_temps(temp_state);
 }
 
 
@@ -132,19 +113,15 @@ void run(const Config &config)
 {
 	tmp_sleeptime = sleeptime;
 
-	read_temps_safe(config.sensors());
-
+	read_temps(config.sensors());
 	temp_state.init();
 
 	// Set initial fan level
-	std::vector<Level *>::const_iterator cur_lvl = config.levels().begin();
-	config.fan()->init();
-	while (cur_lvl != --config.levels().end() && (*cur_lvl)->up())
-		cur_lvl++;
-	log(TF_NFY) << temp_state << " -> " <<
-			(*cur_lvl)->str() << flush;
-	config.fan()->set_speed(*cur_lvl);
+	for (auto &fan_config : config.fan_configs())
+		fan_config->init_fanspeed(temp_state);
+	log(TF_NFY) << temp_state << " -> " << config.fan_configs() << flush;
 
+	bool did_something = false;
 	while (likely(!interrupted)) {
 		auto until = std::chrono::steady_clock::now() + tmp_sleeptime;
 
@@ -155,7 +132,7 @@ void run(const Config &config)
 		if (unlikely(interrupted))
 			break;
 
-		read_temps_safe(config.sensors());
+		read_temps(config.sensors());
 
 		if (unlikely(tolerate_errors) > 0)
 			tolerate_errors--;
@@ -163,28 +140,13 @@ void run(const Config &config)
 		if (unlikely(!temp_state.complete()))
 			throw SystemError(MSG_SENSOR_LOST);
 
-		if (unlikely(cur_lvl != --config.levels().end() && (*cur_lvl)->up())) {
-			while (cur_lvl != --config.levels().end() && (*cur_lvl)->up())
-				cur_lvl++;
-			log(TF_INF) << temp_state << " -> " <<
-					(*cur_lvl)->str() << flush;
-			config.fan()->set_speed(*cur_lvl);
-		}
-		else if (unlikely(cur_lvl != config.levels().begin() && (*cur_lvl)->down())) {
-			while (cur_lvl != config.levels().begin() && (*cur_lvl)->down())
-				cur_lvl--;
-			log(TF_INF) << temp_state << " -> " <<
-					(*cur_lvl)->str() << flush;
-			config.fan()->set_speed(*cur_lvl);
-			tmp_sleeptime = sleeptime;
-		}
-		else {
-			config.fan()->ping_watchdog_and_depulse(*cur_lvl);
-#ifdef DEBUG
-			log(TF_DBG) << temp_state << flush;
-#endif
-		}
+		for (auto &fan_config : config.fan_configs())
+			did_something |= fan_config->set_fanspeed(temp_state);
 
+		if (unlikely(did_something))
+			log(TF_NFY) << temp_state << " -> " << config.fan_configs() << flush;
+
+		did_something = false;
 	}
 }
 
@@ -202,7 +164,7 @@ int set_options(int argc, char **argv)
 	while ((opt = getopt(argc, argv, optstring)) != -1) {
 		switch(opt) {
 		case 'h':
-			log(TF_INF) << MSG_TITLE << flush << MSG_USAGE << flush;
+			log(TF_NFY) << MSG_TITLE << flush << MSG_USAGE << flush;
 			return 1;
 #ifdef USE_ATASMART
 		case 'd':
@@ -210,7 +172,7 @@ int set_options(int argc, char **argv)
 			break;
 #endif
 		case 'c':
-			config_files = std::vector<string>({optarg});
+			config_files = vector<string>({optarg});
 			break;
 		case 'q':
 			--Logger::instance().log_lvl();
@@ -286,7 +248,7 @@ int set_options(int argc, char **argv)
 		}
 	}
 	if (depulse > 0)
-		log(TF_INF) << MSG_DEPULSE(depulse, sleeptime.count()) << flush;
+		log(TF_NFY) << MSG_DEPULSE(depulse, sleeptime.count()) << flush;
 
 	return 0;
 }
@@ -364,7 +326,9 @@ void TemperatureState::restart()
 
 void TemperatureState::add_temp(int t)
 {
-	int diff = t - *temp_;
+	int diff = *temp_ >= 0 ?
+		t - *temp_
+		: 0;
 	*temp_ = t;
 
 	if (unlikely(diff > 2)) {
@@ -412,15 +376,15 @@ bool TemperatureState::complete() const
 { return temp_ == temps_.end(); }
 
 
-const std::vector<int> & TemperatureState::biased_temps() const
+const vector<int> & TemperatureState::biased_temps() const
 { return biased_temps_; }
 
 
-const std::vector<int> & TemperatureState::temps() const
+const vector<int> & TemperatureState::temps() const
 { return temps_; }
 
 
-const std::vector<float> & TemperatureState::biases() const
+const vector<float> & TemperatureState::biases() const
 { return biases_; }
 
 
@@ -439,7 +403,7 @@ int main(int argc, char **argv) {
 
 	struct sigaction handler;
 #if defined(PID_FILE)
-	std::unique_ptr<PidFileHolder> pid_file;
+	unique_ptr<PidFileHolder> pid_file;
 #endif
 
 	if (!isatty(fileno(stdout))) {
@@ -489,12 +453,11 @@ int main(int argc, char **argv) {
 				// Test the config before forking
 				LogLevel old_lvl = Logger::instance().log_lvl();
 				Logger::instance().log_lvl() = TF_ERR;
-				std::unique_ptr<const Config> test_cfg(Config::read_config(config_files));
+				unique_ptr<const Config> test_cfg(Config::read_config(config_files));
 				temp_state = TemperatureState(test_cfg->num_temps());
 				temp_state.init();
-				test_cfg->fan()->init();
-				for (auto &sensor : test_cfg->sensors())
-					sensor->read_temps();
+				test_cfg->init_fans();
+				read_temps(test_cfg->sensors());
 				Logger::instance().log_lvl() = old_lvl;
 				// Own scope so the config gets destroyed before forking
 			}
@@ -505,7 +468,7 @@ int main(int argc, char **argv) {
 				error<SystemError>("Can't fork(): " + msg);
 			}
 			else if (child_pid > 0) {
-				log(TF_INF) << "Daemon PID: " << child_pid << flush;
+				log(TF_NFY) << "Daemon PID: " << child_pid << flush;
 				return 0;
 			}
 			else {
@@ -524,18 +487,18 @@ int main(int argc, char **argv) {
 #endif
 
 		// Load the config for real after forking & enabling syslog
-		std::unique_ptr<const Config> config(Config::read_config(config_files));
+		unique_ptr<const Config> config(Config::read_config(config_files));
 
 		temp_state = TemperatureState(config->num_temps());
 
 		do {
-			config->fan()->init();
+			config->init_fans();
 			run(*config);
 
 			if (interrupted == SIGHUP) {
-				log(TF_INF) << MSG_RELOAD_CONF << flush;
+				log(TF_NFY) << MSG_RELOAD_CONF << flush;
 				try {
-					std::unique_ptr<const Config> config_new(Config::read_config(config_files));
+					unique_ptr<const Config> config_new(Config::read_config(config_files));
 					config.swap(config_new);
 				} catch(ExpectedError &) {
 					log(TF_ERR) << MSG_CONF_RELOAD_ERR << flush;
@@ -546,17 +509,17 @@ int main(int argc, char **argv) {
 				interrupted = 0;
 			}
 			else if (interrupted == SIGUSR2) {
-				config->fan()->init();
+				config->init_fans();
 				interrupted = 0;
 			}
 		} while (!interrupted);
 
-		log(TF_INF) << MSG_TERM << flush;
+		log(TF_NFY) << MSG_TERM << flush;
 #if not defined(DISABLE_EXCEPTION_CATCHING)
 	}
 	catch (InvocationError &e) {
 		log(TF_ERR) << e.what() << flush;
-		log(TF_INF) << MSG_USAGE << flush;
+		log(TF_NFY) << MSG_USAGE << flush;
 	}
 	catch (ExpectedError &e) {
 		log(TF_ERR) << e.what() << flush;
